@@ -2,31 +2,37 @@
 Implementation of the base classes for the ORSO header.
 """
 
-# author: Andrew R. McCluskey (arm61)
-import os.path
-from copy import deepcopy
-from collections.abc import Mapping
-from typing import Optional, Union, List, Tuple, TextIO, Generator, Any
-from inspect import isclass
-from dataclasses import field, dataclass, fields, \
-    _set_new_attribute, _field_init, _create_fn, _init_param, \
-    _FIELD, _FIELDS, _FIELD_INITVAR, MISSING, _HAS_DEFAULT_FACTORY, _POST_INIT_NAME
 import datetime
-import pathlib
-import warnings
 import json
-import yaml
-import yaml.constructor
-from contextlib import contextmanager
+import os.path
+import pathlib
 import re
+import sys
+import warnings
+
+from collections.abc import Mapping
+from contextlib import contextmanager
+from copy import deepcopy
+from dataclasses import (_FIELD, _FIELD_INITVAR, _FIELDS, _HAS_DEFAULT_FACTORY, _POST_INIT_NAME, MISSING, _create_fn,
+                         _field_init, _init_param, _set_new_attribute, dataclass, field, fields)
+from inspect import isclass
+from typing import Any, Generator, List, Optional, TextIO, Tuple, Union
 
 import numpy as np
+import yaml
+import yaml.constructor
 
 # typing stuff introduced in python 3.8
 try:
     from typing import Literal, get_args, get_origin
 except ImportError:
     from .typing_backport import Literal, get_args, get_origin
+# change of signature introduced in python 3.10.1
+if sys.version_info >= (3, 10, 1):
+    _field_init_real = _field_init
+
+    def _field_init(f, frozen, locals, self_name):
+        return _field_init_real(f, frozen, locals, self_name, False)
 
 
 def _noop(self, *args, **kw):
@@ -47,8 +53,9 @@ def __datetime_representer(dumper, data):
 yaml.add_representer(datetime.datetime, __datetime_representer)
 
 # make sure that datetime strings get loaded as str not datetime instances
-yaml.constructor.SafeConstructor.yaml_constructors[u'tag:yaml.org,2002:timestamp'] = (
-    yaml.constructor.SafeConstructor.yaml_constructors[u'tag:yaml.org,2002:str'])
+yaml.constructor.SafeConstructor.yaml_constructors[
+    "tag:yaml.org,2002:timestamp"
+] = yaml.constructor.SafeConstructor.yaml_constructors["tag:yaml.org,2002:str"]
 
 
 def _custom_init_fn(fieldsarg, frozen, has_post_init, self_name, globals):
@@ -62,14 +69,10 @@ def _custom_init_fn(fieldsarg, frozen, has_post_init, self_name, globals):
             if not (f.default is MISSING and f.default_factory is MISSING):
                 seen_default = True
             elif seen_default:
-                raise TypeError(f'non-default argument {f.name!r} '
-                                'follows default argument')
+                raise TypeError(f"non-default argument {f.name!r} " "follows default argument")
 
-    locals = {f'_type_{f.name}': f.type for f in fieldsarg}
-    locals.update({
-        'MISSING': MISSING,
-        '_HAS_DEFAULT_FACTORY': _HAS_DEFAULT_FACTORY,
-    })
+    locals = {f"_type_{f.name}": f.type for f in fieldsarg}
+    locals.update({"MISSING": MISSING, "_HAS_DEFAULT_FACTORY": _HAS_DEFAULT_FACTORY})
 
     body_lines = []
     for f in fieldsarg:
@@ -78,60 +81,56 @@ def _custom_init_fn(fieldsarg, frozen, has_post_init, self_name, globals):
             body_lines.append(line)
 
     if has_post_init:
-        params_str = ','.join(f.name for f in fieldsarg
-                              if f._field_type is _FIELD_INITVAR)
-        body_lines.append(f'{self_name}.{_POST_INIT_NAME}({params_str})')
+        params_str = ",".join(f.name for f in fieldsarg if f._field_type is _FIELD_INITVAR)
+        body_lines.append(f"{self_name}.{_POST_INIT_NAME}({params_str})")
 
     # processing of additional user keyword arguments
-    body_lines += ['for k,v in user_kwds.items():', '    setattr(self, k, v)']
+    body_lines += ["for k,v in user_kwds.items():", "    setattr(self, k, v)"]
 
-    return _create_fn('__init__',
-                      [self_name] + [_init_param(f) for f in fieldsarg if f.init] + ['**user_kwds'],
-                      body_lines,
-                      locals=locals,
-                      globals=globals,
-                      return_type=None)
+    return _create_fn(
+        "__init__",
+        [self_name] + [_init_param(f) for f in fieldsarg if f.init] + ["**user_kwds"],
+        body_lines,
+        locals=locals,
+        globals=globals,
+        return_type=None,
+    )
 
 
-class _HeaderMeta(type):
-    """
-    Metaclass for :py:class:`Header`.
-    Creates a :py:attr:`dataclass` with an additional comment attribute.
-    """
+def orsodataclass(cls: type):
+    attrs = cls.__dict__
+    bases = cls.__bases__
+    if "__annotations__" in attrs and len([k for k in attrs["__annotations__"].keys() if not k.startswith("_")]) > 0:
+        # only applies to dataclass children of Header
+        # add optional comment attribute, needs to come last
+        attrs["__annotations__"]["comment"] = Optional[str]
+        setattr(cls, "comment", field(default=None))
 
-    def __new__(cls, name, bases, attrs, **kwargs):
-        if '__annotations__' in attrs and \
-                len([k for k in attrs['__annotations__'].keys() if not k.startswith('_')]) > 0:
-            # only applies to dataclass children of Header
-            # add optional comment attribute, needs to come last
-            attrs['__annotations__']['comment'] = Optional[str]
-            attrs['comment'] = field(default=None)
+        # create the _orso_optional attribute
+        orso_optionals = []
+        for fname, ftype in attrs["__annotations__"].items():
+            if type(None) in get_args(ftype):
+                orso_optionals.append(fname)
+        for base in bases:
+            if hasattr(base, "_orso_optionals"):
+                orso_optionals += getattr(base, "_orso_optionals")
+        setattr(cls, "_orso_optionals", orso_optionals)
+        out = dataclass(cls, repr=False, init=False)
+        fieldsarg = getattr(out, _FIELDS)
 
-            # create the _orso_optional attribute
-            attrs['_orso_optionals'] = []
-            for fname, ftype in attrs['__annotations__'].items():
-                if type(None) in get_args(ftype):
-                    attrs['_orso_optionals'].append(fname)
-            for base in bases:
-                if hasattr(base, '_orso_optionals'):
-                    attrs['_orso_optionals'] += getattr(base, '_orso_optionals')
-            out = dataclass(type.__new__(cls, name, bases, attrs, **kwargs), repr=False, init=False)
-            fieldsarg = getattr(out, _FIELDS)
+        # Generate custom __init__ method that allows arbitrary extra keyword arguments
+        has_post_init = hasattr(out, _POST_INIT_NAME)
+        # Include InitVars and regular fields (so, not ClassVars).
+        flds = [f for f in fieldsarg.values() if f._field_type in (_FIELD, _FIELD_INITVAR)]
+        init_fun = _custom_init_fn(flds, False, has_post_init, "self", globals())
+        _set_new_attribute(out, "__init__", init_fun)
 
-            # Generate custom __init__ method that allows arbitrary extra keyword arguments
-            has_post_init = hasattr(out, _POST_INIT_NAME)
-            # Include InitVars and regular fields (so, not ClassVars).
-            flds = [f for f in fieldsarg.values()
-                    if f._field_type in (_FIELD, _FIELD_INITVAR)]
-            init_fun = _custom_init_fn(flds, False, has_post_init, 'self', globals())
-            _set_new_attribute(out, '__init__', init_fun)
-
-            return out
-        else:
-            return type.__new__(cls, name, bases, attrs, **kwargs)
+        return out
+    else:
+        return cls
 
 
-class Header(metaclass=_HeaderMeta):
+class Header:
     """
     The super class for all of the items in the orso module.
     """
@@ -155,12 +154,12 @@ class Header(metaclass=_HeaderMeta):
                         f"No suitable conversion found for {fld.name}, "
                         f"{fld.type} with value {attr}, "
                         "setting attribute with raw value from ORSO file",
-                        RuntimeWarning
+                        RuntimeWarning,
                     )
                     setattr(self, fld.name, attr)
                     # raise ValueError(f"No suitable conversion found for {fld.type} with value {attr}")
 
-        if hasattr(self, 'unit'):
+        if hasattr(self, "unit"):
             self._check_unit(self.unit)
 
     @property
@@ -188,7 +187,7 @@ class Header(metaclass=_HeaderMeta):
         :return: Correctly resolved object with required type for orso
             compatibility.
         """
-        if isclass(hint) and not getattr(hint, '__origin__', None) in [List, Tuple, Union, Literal]:
+        if isclass(hint) and not getattr(hint, "__origin__", None) in [List, Tuple, Union, Literal]:
             # simple type that we can work with, no Union or List/Dict
             if isinstance(item, hint):
                 return item
@@ -198,8 +197,8 @@ class Header(metaclass=_HeaderMeta):
                     return datetime.datetime.fromisoformat(item)
                 except AttributeError:  # python 3.6
                     try:
-                        item = item.split('+', 1)[0]
-                        if '.' in item:
+                        item = item.split("+", 1)[0]
+                        if "." in item:
                             return datetime.datetime.strptime(item, "%Y-%m-%dT%H:%M:%S.%f")
                         else:
                             return datetime.datetime.strptime(item, "%Y-%m-%dT%H:%M:%S")
@@ -212,17 +211,13 @@ class Header(metaclass=_HeaderMeta):
             if issubclass(hint, Header):
                 # convert to dataclass instance
                 attribs = hint.__annotations__.keys()
-                realised_items = {
-                    k: item[k] for k in item.keys() if k in attribs
-                }
+                realised_items = {k: item[k] for k in item.keys() if k in attribs}
                 # orphan_items are extra dictionary entries that don't correspond to
                 # arguments of a Header instance. They can't be used to construct
                 # a header instance, so remove them from the dict of arguments.
                 # This enables back compatibility of Orso files as attributes
                 # present in later versions will still be loadable by earlier code.
-                orphan_items = {
-                    k: item[k] for k in item.keys() if k not in attribs
-                }
+                orphan_items = {k: item[k] for k in item.keys() if k not in attribs}
 
                 try:
                     value = hint(**realised_items)
@@ -244,9 +239,7 @@ class Header(metaclass=_HeaderMeta):
             if hbase in (list, tuple):
                 t0 = get_args(hint)[0]
                 if isinstance(item, (list, tuple)):
-                    return type(item)(
-                        [Header._resolve_type(t0, i) for i in item]
-                    )
+                    return type(item)([Header._resolve_type(t0, i) for i in item])
                 else:
                     return [Header._resolve_type(t0, item)]
             elif hbase in [Union, Optional]:
@@ -266,7 +259,7 @@ class Header(metaclass=_HeaderMeta):
         return None
 
     @classmethod
-    def empty(cls) -> 'Header':
+    def empty(cls) -> "Header":
         """
         Create an empty instance of this item containing
         all non-option attributes as :code:`None`.
@@ -282,15 +275,18 @@ class Header(metaclass=_HeaderMeta):
                 attr_items[fld.name] = fld.type.empty()
             elif get_origin(fld.type) is Union and issubclass(get_args(fld.type)[0], Header):
                 attr_items[fld.name] = get_args(fld.type)[0].empty()
-            elif get_origin(fld.type) is list and isclass(get_args(fld.type)[0]) \
-                    and issubclass(get_args(fld.type)[0], Header):
+            elif (
+                get_origin(fld.type) is list
+                and isclass(get_args(fld.type)[0])
+                and issubclass(get_args(fld.type)[0], Header)
+            ):
                 attr_items[fld.name] = [get_args(fld.type)[0].empty()]
             else:
                 attr_items[fld.name] = None
         return cls(**attr_items)
 
     @staticmethod
-    def asdict(header: 'Header') -> dict:
+    def asdict(header: "Header") -> dict:
         """
         Static method for :py:func:`to_dict`.
 
@@ -309,9 +305,7 @@ class Header(metaclass=_HeaderMeta):
         """
         out_dict = {}
         for i, value in self.__dict__.items():
-            if i.startswith("_") or (
-                    value is None and i in self._orso_optionals
-            ):
+            if i.startswith("_") or (value is None and i in self._orso_optionals):
                 continue
 
             if hasattr(value, "to_dict"):
@@ -350,19 +344,19 @@ class Header(metaclass=_HeaderMeta):
         """
         if unit is not None:
             # raise UnicodeError if not ascii
-            unit.encode('ascii')
+            unit.encode("ascii")
 
     def __repr__(self):
         """
         Representation that does not show empty arguments.
         """
-        out = f'{self.__class__.__name__}('
+        out = f"{self.__class__.__name__}("
         for fi in fields(self):
             if fi.name in self._orso_optionals and getattr(self, fi.name) is None:
                 # ignore empty optional arguments
                 continue
-            out += f'{fi.name}={getattr(self, fi.name)!r}, '
-        out = out[:-2] + ')'
+            out += f"{fi.name}={getattr(self, fi.name)!r}, "
+        out = out[:-2] + ")"
         return out
 
     def _staggered_repr(self):
@@ -373,30 +367,30 @@ class Header(metaclass=_HeaderMeta):
         To use in a subclass, the __repr__ method has to be replaced with this one.
         """
         slen = len(self.__class__.__name__)
-        out = f'{self.__class__.__name__}(\n'
+        out = f"{self.__class__.__name__}(\n"
         for fi in fields(self):
             if fi.name in self._orso_optionals and getattr(self, fi.name) is None:
                 # ignore empty optional arguments
                 continue
             nlen = len(fi.name)
             ftxt = repr(getattr(self, fi.name))
-            ftxt = ftxt.replace('\n', '\n' + ' ' * (slen + nlen + 2))
-            out += ' ' * (slen + 1) + f'{fi.name}={ftxt},\n'
-        out += ' ' * (slen + 1) + ')'
+            ftxt = ftxt.replace("\n", "\n" + " " * (slen + nlen + 2))
+            out += " " * (slen + 1) + f"{fi.name}={ftxt},\n"
+        out += " " * (slen + 1) + ")"
         return out
 
 
+@orsodataclass
 class Value(Header):
     """
     A value or list of values with an optional unit.
     """
 
     magnitude: Union[float, List[float]]
-    unit: Optional[str] = field(
-        default=None, metadata={"description": "SI unit string"}
-    )
+    unit: Optional[str] = field(default=None, metadata={"description": "SI unit string"})
 
 
+@orsodataclass
 class ValueRange(Header):
     """
     A range or list of ranges with mins, maxs, and an optional unit.
@@ -404,11 +398,10 @@ class ValueRange(Header):
 
     min: Union[float, List[float]]
     max: Union[float, List[float]]
-    unit: Optional[str] = field(
-        default=None, metadata={"description": "SI unit string"}
-    )
+    unit: Optional[str] = field(default=None, metadata={"description": "SI unit string"})
 
 
+@orsodataclass
 class ValueVector(Header):
     """
     A vector or list of vectors with an optional unit.
@@ -426,11 +419,10 @@ class ValueVector(Header):
     x: Union[float, List[float]]
     y: Union[float, List[float]]
     z: Union[float, List[float]]
-    unit: Optional[str] = field(
-        default=None, metadata={"description": "SI unit string"}
-    )
+    unit: Optional[str] = field(default=None, metadata={"description": "SI unit string"})
 
 
+@orsodataclass
 class Person(Header):
     """
     Information about a person, including name, affilation(s), and contact
@@ -439,25 +431,21 @@ class Person(Header):
 
     name: str
     affiliation: Union[str, List[str]]
-    contact: Optional[str] = field(
-        default=None, metadata={"description": "Contact (email) address"}
-    )
+    contact: Optional[str] = field(default=None, metadata={"description": "Contact (email) address"})
 
 
+@orsodataclass
 class Column(Header):
     """
     Information about a data column.
     """
 
     name: str
-    unit: Optional[str] = field(
-        default=None, metadata={"description": "SI unit string"}
-    )
-    dimension: Optional[str] = field(
-        default=None, metadata={"dimension": "A description of the column"}
-    )
+    unit: Optional[str] = field(default=None, metadata={"description": "SI unit string"})
+    dimension: Optional[str] = field(default=None, metadata={"dimension": "A description of the column"})
 
 
+@orsodataclass
 class File(Header):
     """
     A file with file path and a last modified timestamp.
@@ -468,8 +456,8 @@ class File(Header):
         default=None,
         metadata={
             "description": "Last modified timestamp. If it's not specified,"
-                           " then an attempt will be made to check on the file"
-                           " itself"
+            " then an attempt will be made to check on the file"
+            " itself"
         },
     )
 
@@ -481,9 +469,7 @@ class File(Header):
         if self.timestamp is None:
             fname = pathlib.Path(self.file)
             if fname.exists():
-                self.timestamp = datetime.datetime.fromtimestamp(
-                    fname.stat().st_mtime
-                )
+                self.timestamp = datetime.datetime.fromtimestamp(fname.stat().st_mtime)
 
 
 def _read_header_data(file: Union[TextIO, str], validate: bool = False) -> Tuple[List[dict], list, str]:
@@ -517,7 +503,7 @@ def _read_header_data(file: Union[TextIO, str], validate: bool = False) -> Tuple
         for i, line in enumerate(fi.readlines()):
             if not line.startswith("#"):
                 # ignore empty lines
-                if line.strip() != '':
+                if line.strip() != "":
                     _ds_lines.append(line)
                 continue
 
@@ -528,9 +514,7 @@ def _read_header_data(file: Union[TextIO, str], validate: bool = False) -> Tuple
                 # a new dataset is starting. Complete the previous dataset's
                 # numerical array  and start collecting the numbers for this
                 # dataset
-                _d = np.array(
-                    [np.fromstring(v, dtype=float, sep=" ") for v in _ds_lines]
-                )
+                _d = np.array([np.fromstring(v, dtype=float, sep=" ") for v in _ds_lines])
                 data.append(_d)
                 _ds_lines = []
 
@@ -543,9 +527,7 @@ def _read_header_data(file: Union[TextIO, str], validate: bool = False) -> Tuple
                 header.append(line[1:])
 
         # append the last numerical array
-        _d = np.array(
-            [np.fromstring(v, dtype=float, sep=" ") for v in _ds_lines]
-        )
+        _d = np.array([np.fromstring(v, dtype=float, sep=" ") for v in _ds_lines])
         data.append(_d)
 
         yml = "".join(header)
@@ -557,9 +539,7 @@ def _read_header_data(file: Union[TextIO, str], validate: bool = False) -> Tuple
         )
 
         if not pattern.match(header[0].lstrip(" ")):
-            raise ValueError(
-                "First line does not appear to match that of an ORSO file"
-            )
+            raise ValueError("First line does not appear to match that of an ORSO file")
         version = re.findall(r"([0-9]+\.?[0-9]*|\.[0-9]+)+?", header[0])[0]
 
         dcts = yaml.safe_load_all(yml)
@@ -594,6 +574,7 @@ def _validate_header_data(dct_list: List[dict]):
         :code:`'sQz'` are not the same.
     """
     import jsonschema
+
     req_cols = ["Qz", "R", "sR", "sQz"]
     acceptable_Qz_units = ["1/angstrom", "1/nm"]
 
@@ -612,8 +593,8 @@ def _validate_header_data(dct_list: List[dict]):
         cols = dct["columns"]
 
         ncols = min(4, len(cols))
-        col_names = [col['name'] for col in cols]
-        units = [col.get('unit') for col in cols]
+        col_names = [col["name"] for col in cols]
+        units = [col.get("unit") for col in cols]
 
         # If dct was created with Orso.empty() the Orso.columns attribute
         # will be [{"name": None}]. In which case don't both validating column
@@ -622,14 +603,10 @@ def _validate_header_data(dct_list: List[dict]):
             continue
 
         if col_names[:ncols] != req_cols[:ncols]:
-            raise ValueError(
-                "The first four columns should be named"
-                " 'Qz', 'R', ['sR', ['sQz']]"
-            )
+            raise ValueError("The first four columns should be named" " 'Qz', 'R', ['sR', ['sQz']]")
 
         if units[0] not in acceptable_Qz_units:
-            raise ValueError("The Qz column must have units of '1/angstrom'"
-                             " or '1/nm'")
+            raise ValueError("The Qz column must have units of '1/angstrom'" " or '1/nm'")
 
         if len(units) >= 4 and units[0] != units[3]:
             raise ValueError("Columns Qz and sQz must have the same units'")
@@ -682,9 +659,11 @@ def _todict(obj: Any, classkey: Any = None) -> dict:
         return [_todict(v, classkey) for v in obj]
     elif hasattr(obj, "__dict__"):
         data = dict(
-            [(key, _todict(value, classkey)) for key, value
-             in obj.__dict__.items()
-             if not callable(value) and not key.startswith('_')]
+            [
+                (key, _todict(value, classkey))
+                for key, value in obj.__dict__.items()
+                if not callable(value) and not key.startswith("_")
+            ]
         )
 
         if classkey is not None and hasattr(obj, "__class__"):
